<?php
class WP_YogaTimeTable_HtmlRenderer extends WP_YogaTimeTable_Renderer {

	protected $_wpytt;
	protected $_opts;
	protected $_css;
	protected $_teachersControlsData;
	protected $_levelsControlsData;
	protected $_intervals;
	protected $_breakHours;
	protected $_intervalsMask;
	protected $_bitsPerHour;
	protected $_days;
	
	function __construct( $templater, $wpytt, $wpyttOpts, $wpyttCss ) {
		$this->_templater = $templater;
		$this->_wpytt = $wpytt;
		$this->_opts = $wpyttOpts;
		$this->_css = $wpyttCss;

		$this->_teachersControlsData = $this->_templater->getGenericBitmaskAndControls( $this->_templater->getTeachers(), 'cbprof' );
		$this->_levelsControlsData = $this->_templater->getGenericBitmaskAndControls( WP_Plugin_FormHelper::wpytt_get_niveaux_kva(''), 'cbniveau' );

	}
	
	/* Get the hours "head" column */
	function renderHeadColumn() {

		$hmin = $this->_opts['heure_mini'] / 60;
		$hmax = $this->_opts['heure_maxi'] / 60;
		$span = $hmax - $hmin;
//		echo "<!-- $hmin, $hmax -->\n";

		$idx_int = 0;
		
		$hours_column = "<div class='vday vday_narrow'><!--\n" .
			"\t--><div class='vyclass colhead'>Heure</div><!--\n";

		for( $i = 0 ; $i < $span ; $i++ ) {
//			echo "<!-- $i -->\n";
			$intervalle = $this->_intervals[$idx_int];
			if( $i == $intervalle['debut'] ) {
				$sep =  ' vsep';
				$heure = $intervalle['debut'] . '-' . $intervalle['fin'] + $hmin;
				$idx_int++;
				$i += $intervalle['duree'] - 1;
			}
			else {
				$sep = '';
				$heure = $i + $hmin;
			}
			$hours_column .= "\t--><div class='vhour{$sep}'>&nbsp;$heure</div><!--\n";
		}

		$hours_column .= "--></div>\n";
		return $hours_column;
	}
	
	function computeMargin( $class, $previous, $intervals ) {
		$startDayMinutes = $this->_opts['heure_mini'];
		// Get previous class end
		$previousClassEndMinutes = $previous != null ? $previous['endHour'] - $startDayMinutes : 0;
		$previousClassEndHours = floor( $previousClassEndMinutes / 60 );
		// Get this class start
		$thisClassBeginMinutes = $class['stHour'] - $startDayMinutes;
		$thisClassBeginHours = floor( $thisClassBeginMinutes / 60 );
		$thisClassEndMinutes = $class['endHour'] - $startDayMinutes;
		$thisClassEndHours = floor( $thisClassEndMinutes / 60 );
		// Get intervals	
		$numBreaks = 0; $numBreakHrs = 0;
		if( $intervals != null )
			foreach( $intervals as $i => $break ) {
				
				// Compare interval hours with class hours
				// en comparant heures de debut et fin des deux
				
				if( $break['debut'] >= $thisClassBeginHours ) 
					break;
				if( $break['fin'] <= $previousClassEndHours )
					break;
					$numBreaks++;
					$numBreakHrs += $break['duree'];
				
			}
		// Time interval between previous class and current class,
		// then substract break intervals total duration from this value
		$marginMinutes = $thisClassBeginMinutes - $previousClassEndMinutes - ( 60 * $numBreakHrs );
		// Compute margin in pixels
		$marginPx = $numBreaks * $this->_css['sep_height'] + ( $marginMinutes * $this->_css['hour_height'] ) / 60;
				
		return $marginPx != 0 ? $marginPx - 1 : $marginPx;
	}
	
	/**
	 * Output a table column, corresponding to a day of the week
	 * @param day the day of week
	 */
	function renderColumn( $day ) {
	
		$dayPretty = $this->_days[$day];

		$output = "\n\n<div class='vday vday-{$day}'><!--\n"
			."--><div class='vyclass colhead'><b>{$dayPretty}</b></div><!--\n";
		
		$classesByDay = $this->_templater->getClasses();
		$days = $this->_templater->getDays();
		$classes = $classesByDay[$day];
		$idx = 0;
		$previousClass = null;
		foreach( $classes as $classSlug => $classData ) {
			extract( $classData );
			// TODO: virer getTermNiveaux
			// getTermNiveaux returns ( $niveaux, $niveaux_str, $premier_niv_str, $slugs_str, $nivs_mask );
			list( $levels, $levelsString, $firstLevelString, $levelSlugs, $levelsMask ) = $this->_templater->getTermNiveaux( $id );
			$classData['levelsString'] = $levelsString;
			$classData['firstLevelString'] = $firstLevelString;
			$teachersMask = $this->_teachersControlsData[0][$teacherSl];

			$intervals = $this->_intervals;
			$breakHrs = $this->_breakHours;
			
			$marginMinutes = $idx++ == 0 ? $stHour - $this->_opts['heure_mini'] : $stHour - $previousEndHour;
			$duration = $endHour - $stHour;

			$heightPx = ( $duration * $this->_css['hour_height'] ) / 60 - ( $marginPx == 0 ? 2 : 1 );
			$marginPx = $this->computeMargin( $classData, $previousClass, $intervals );
			$style = "margin-top: {$marginPx}px; height: {$heightPx}px;";
			$style .= $marginPx == 0 ? ' border-top: 0;' : '';
			$textStyle = "vertical-align: bottom;";
			
			$boxModel = $this->_templater->getBoxModel( $model );
			if( empty( $boxModel ) ) $boxModel = $this->_templater->getBoxModel( 'standard' );
			//customDump( $boxModel );
			$boxParams = array();
			foreach( $boxModel['args'] as $arg )
				$boxParams[] = $classData[$arg];
			$boxContent = vsprintf( $boxModel['string'], $boxParams );
			
			$isAdmin = current_user_can( 'manage_options' );
			$linkOpen = $isAdmin ? "<a href='" . get_edit_post_link( $id ) . "'>" : "";
			$linkClose = $isAdmin ? "</a>" : "";
			$boxContent = str_replace( '{linkOpen}', $linkOpen, $boxContent );
			$boxContent = str_replace( '{linkClose}', $linkClose, $boxContent );
			
			$output .= "--><div class='vyclass $teacherSl $levelSlugs' style='$style'"
				  . "><input type='hidden' value='$teachersMask|$levelsMask' /><div class='class-text'>"
				  . $boxContent
				  //. "$title<br>$stHourPr &mdash; $endHourPr"
				  . "</div></div><!--\n";
			$previousClass = $classData;
			
		}
		$output .= "--></div>\n\n";
		return $output;
	}
	
	function renderIntervalsColumn() {
		$out = "<div class='vday vday-'><!--"
			 . "\t--><div class='vyclass colhead'>Mask</div><!--\n";

		//list( $intervals, $breakHrs, $mask, $bph ) = $this->_templater->computeGaps();
		$mask = $this->_intervalsMask;
		$bph = $this->_bitsPerHour;
		$bphm = $bph - 1;
		$bitHeightPx = floor( $this->_css['hour_height'] / $bph );
		$i = 0;
		foreach( $mask as $bit ) {
			$out .= $i % $bph == 0 ? "--><div class='vhour' style='border-bottom: 1px solid gray;'>" : "";
			
			$color = $bit ? 'green' : 'gray';
			$out .= "<div style='height: {$bitHeightPx}px; width: 100%; background: $color;'></div>" ;
			$out .= $i % $bph == $bphm ? "</div><!--" : "";
			$i++;
			}
		$out .= "--></div>\n\n";
		return $out;
	}

	/* Get the timetable JS controls */
	function renderControls() {
		$controls = "\n\n<div style='padding: 4px; font-size: 11px;'><span class='cb-row-head'>Professeurs&nbsp;:</span>\n";
		$controls .= implode( "\n", $this->_teachersControlsData[1] );
		$teachersMaskAll = $tthis->_teachersControlsData[0]['__ALL__'];

		$controls .= "<br><input type='hidden' id='pmask' name='pmask' value='$teachersMaskAll' />\n" .
			"<span class='cb-row-head'>Niveaux&nbsp;:</span>\n";
		$controls .= implode( "\n", $this->_levelsControlsData[1] );
		$levelsMaskAll = $this->_levelsControlsData[0]['__ALL__'];
		
		$controls .= "<br><input type='hidden' id='nmask' name='nmask' value='$levelsMaskAll' />\n";
		
		return $controls;
		}
	
	function renderHeader() {
		$out = $this->renderControls();
		$out .= '<div class="vtt-box">';
		return $out;
	}
	
	function renderFooter() {
		return "</div>\n<!-- vtt-box -->\n</div>\n";
	}

	function renderInit() {
		/* Get intervals */
		list( $intervals, $breakHours, $mask, $bph ) = $this->_templater->computeGaps();
		$this->_intervals = $intervals;
		$this->_breakHours = $breakHours;
		$this->_intervalsMask = $mask;
		$this->_bitsPerHour = $bph;
	}
	

}
?>