<?php
interface WPYTT_TableIO {
	function writeTable( $table );
	function readTable();
}

class WPYTT_FileTableIO implements WPYTT_TableIO {
	
	private $_filePath;
	private $_fileName;
	
	function __construct( $path, $file ) {
		$this->_filePath = $path;
		$this->_fileName = $file;
	}
	
	function writeTable( $table ) {
		$fh = fopen( $this->_filePath . $this->_fileName, 'w+' );
		if( !$fh ) die( 'die on file open' );
		$fw = fwrite( $fh, $table );
		if( !$fw ) die( 'die on file write' );
		$fcSuccess = fclose( $fh );
	}

	function fileErrorHandler($errno, $errstr, $errfile, $errline) {
		die('I/O error');
	}
	
	function readTable() {
		$oldHandler = set_error_handler( array( $this, 'fileErrorHandler' ) );
		$fileName = $this->_filePath . $this->_fileName;
		$fh = fopen( $fileName, 'r' );
		$contents = fread( $fh, filesize( $fileName ) );
		$fcSuccess = fclose( $fh );
		restore_error_handler();
		return ( !$fh || !$contents || $fsSuccess ) ? false : $contents;
	}

}

class WPYTT_DBaseTableIO implements WPYTT_TableIO {
	
	private $_filePath;
	private $_fileName;
	
	function __construct() {
	}
	
	function writeTable( $table ) {
		// Create post object
		  $tablePost = array(
			 'post_title' => 'Time Table ' . date('Y-m-d H:i'),
			 'post_status' => 'publish',
			 'post_author' => wp_get_current_user()->ID,
			 'post_content' => $table,
			 'post_type' => 'time_table'
		  );

		// Insert the post into the database
		wp_insert_post( $tablePost );
	}

	function readTable() {
		$tablePosts = get_posts( array( 
			'post_type' => 'time_table', 'numberposts' => 1, 'post_status' => 'publish', 
			'orderby' => 'post_date', 'order' => 'DESC' ) );
		if( empty( $tablePosts ) ) return 'Not found !';
		$tablePost = $tablePosts[0];
//		customEcho( "Retrieved from DB : " . $tablePost->ID . ", " . $tablePost->post_title, 5 );
		return $tablePost->post_content;
	}

}
?>