<?php
require( 'classes/wpytt-renderer.php' );
require( 'classes/wpytt-eps-renderer.php' );
require( 'classes/wpytt-html-renderer.php' );
require( 'classes/wpytt-table-io.php' );

/**
 * 
 * Utility function to echo strings
 * @param string $str	string to echo
 * @param int $level	title level
 */
function customEcho( $str, $level = 0 ) {
	$openTag = $level > 0 ? "<h{$level}>" : "";
	$closeTag = $level > 0 ? "</h{$level}>" : "";
	echo "{$openTag}{$str}{$closeTag}<br>";
}

/**
 * 
 * Dump an array in HTML readable form
 * @param unknown_type $mixed	array to dump
 * @param unknown_type $level	recursion level
 */
function customDump( $mixed, $level = 0 ) {
	$i = 0;
	$indent = $level * 20;
	if( is_array( $mixed ) )
		foreach( $mixed as $key => $val ) {
			echo "<span style='margin-left: {$indent}px;'>";
			echo "k:$key => v:";
			if( is_array( $val ) ) {
				echo "<br>"; customDump( $val, $level + 1 );
			}
			else echo "$val<br>";
			echo "</span><br>";
		}
	else {
		echo "<span style='margin-left: {$indent}px;'>";
		echo "$i -> $mixed<br>";
		echo "</span><br>";
	}
}
	

class WP_YogaTimeTable_Templater {

	protected $_wpytt;
	protected $_opts;
	protected $_css;
	protected $_renderer;
	protected $_filePath;
	protected $_fileIo;
	protected $_dbIo;
	
	function __construct( $wpytt ) {
		$this->_wpytt = $wpytt;
		$this->_opts = $wpytt->settings['wpytt_options'];
		$this->_css = $wpytt->settings['wpytt_options_css'];
		$this->_renderer = new WP_YogaTimeTable_HtmlRenderer( $this, $wpytt, $this->_opts, $this->_css );
		$this->_filePath = $this->_wpytt->getPath();
		$this->_fileIo = new WPYTT_FileTableIO( $this->_filePath, 'timetable.html.part' );
		$this->_dbIo = new WPYTT_DBaseTableIO();
		add_filter( 'the_content', array( $this, 'filterPost' ) );
	}

	function getRenderer() {
		return $this->_renderer;
	}
	
	function regenerateCss( $old, $new ) {

		$cssVars = $new;
		list( $intervals, $numBreakHours, $mask, $bitsPerHour ) = $this->computeGaps();
		
		$cssVars['day_height'] = ( $this->_opts['day_duration'] * $cssVars['hour_height'] ) / 60
			- ( $numBreakHours * $cssVars['hour_height'] ) 
			+ count( $intervals ) * $cssVars['sep_height']
			+ $cssVars['head_cell_height'] + $cssVars['head_cell_padding_top'] + $cssVars['head_cell_border_bottom'];
		
		/* Get colors */
		$couleursNiveaux = $cssVars['term_color'];
		$terms_niveaux = WP_Plugin_FormHelper::wpytt_get_niveaux( 'DESC' );
//		var_dump( $couleursNiveaux );
//		var_dump( $terms_niveaux );
		$css = "/* Colors */\n";
		foreach( $terms_niveaux as $term_niveau ) {
			//echo "term_niv: $term_niveau, ";
			$niveau = $term_niveau->slug;
//			echo "niv: $niveau <br>";
			$couleur = $couleursNiveaux[$niveau];
			$css .= ".$niveau , ._$niveau {"
				 . "\tbackground: #$couleur;} ";
		}

		/* Open CSS template and fill an array with its lines */
		$thisfile = basename( __FILE__ );
		//$thispath = $this->_wpytt->getPath();
		$fh = fopen( $this->_filePath . 'timetable.template.css', 'r' );
		while( $line = fgets( $fh ) ) $css .= $line;
		fclose( $fh );

		foreach( $cssVars as $var => $value ) {

				/* Decrement hours heights */
				if( $var == 'hour_height' || $var == 'sep_height' ) $value -= 1;

				/* Translate font choice  in px */
				if( $var == 'font_size' ) {
					$tailles = WP_Plugin_FormHelper::get_tailles_police();
					$value = $tailles[$value];
				}
		
				$css = str_replace( "[$var]", $value, $css );
			}

		$lines = explode( "\n", $css );
		$fh = fopen( $this->_filePath . 'timetable.css', 'w+' );
		foreach( $lines as $line ) fputs( $fh, $line );
		fclose( $fh );
		
	//	$this->_fileIo->writeTable( $this->_renderer->render( false ) );
		$this->_dbIo->writeTable( $this->_renderer->render( false ) );
	}
	
	function filterPost( $content ) {

		/* Preg match setup */
		$matches = array();
		$ids = array();
		$table_id = 0; // Also set $table_post in future versions ?
		$pattern_to_replace = array();
		
		/* Case 1 : LEFT FOR FUTURE USE : we may have several tables in the future, managed by custom posts */
		if( preg_match( '/\[timetable id=(\d)+\]/', $content, $matches ) == 1 ) {
			// An ID was specified, then let's retrieve it
			preg_match( '/(\d)+/', $matches[0], $ids );
			$table_id = $ids[0];
			// Then get Custom post ---- WARNING !! This won't be a standard WP post but a CUSTOM post
			$table_post = get_post( $table_id );

			$pattern_to_replace[] = '/\[timetable id=(\d)+\]/';
		}
		/* Case 2 : we only have ONE timetable containing all the (yoga) classes */
		else if( preg_match( '/\[timetable]/', $content, $matches ) == 1 )
			$pattern_to_replace[] = '/\[timetable]/';
		else return $content;
		
		$replacement_string = $this->_dbIo->readTable();
		if( empty( $replacement_string ) )
			$replacement_string = $this->_fileIo->readTable();
		if( empty( $replacement_string ) )
			$replacement_string = $this->_renderer->render();
		$replacement_a = array( $replacement_string ); 

	//	$this->_epsRenderer = new WP_YogaTimeTable_EpsRenderer( $this, $wpytt, $this->_opts, $this->_css );
	//	$this->_epsRenderer->render();
		
		return( preg_replace($pattern_to_replace, $replacement_a, $content) ); 
	}

	/* End wpytt_generate_table_controls */

	function getBoxModel( $model ) {
			$models = array(
				// link open, levels string, start hour pretty, end hour pretty, link close, teacher
				"standard" => array(
					"string" => "{linkOpen}%s : %s &mdash; %s{linkClose}<br>%s",
					"args"	 => array( 'levelsString', 'stHourPr', 'endHourPr', 'teacherNm' )
				),
				// link open, levels string, start hour pretty, end hour pretty, link close, teacher
				"multiple" => array(
					"string" => "{linkOpen}%s : d&eacute;but %s{linkClose}<br>%s<br>%s&nbsp;: fin %s",
					"args"	 => array( 'levelsString', 'stHourPr', 'teacherNm', 'firstLevelString', 'endHourPr' )
				),
				"court" => array(
					"string" => "{linkOpen}%s : fin %s{linkClose}",
					"args"	 => array( 'firstLevelString', 'endHourPr' )
				),
				"custom" => array(
					"string" => "{linkOpen}%s.{linkClose}",
					"args"	 => array( 'modelCust' )
				)
			);
			return $models[$model];
				
	}
	
	/* Get the days that are enabled 
	 * Todo: remove duplicate code with wpytt-data-functions.php */
	function getDays() {
		$shownCodes = $this->_opts['jours_visibles'];
		$days = array(	'lundi' => 'Lundi', 'mardi' => 'Mardi', 'mercredi' => 'Mercredi', 
						'jeudi' => 'Jeudi', 'vendredi' => 'Vendredi', 'samedi' => 'Samedi', 'dimanche' => 'Dimanche' );
		$shownDays = array();
		foreach( $shownCodes as $code )
			$shownDays[$code] = $days[$code];
		return $shownDays;
	}


	function getTeachers() {
		$teacherObjects = get_posts( array( 
			'post_type' => 'les-professeurs', 'numberposts' => -1, 'post_status' => 'publish', 
			'exclude' => '361', 'orderby' => 'menu_order', 'order' => 'ASC' ) );
		$shownTeachers = $this->_opts['professeurs_visibles'];
		//if( empty( $shownTeachers ) ) return array();
		$teachers = array();
		foreach( $teacherObjects as $teacherObj ) {
			$slug = $teacherObj->post_name;
			if( !in_array( $slug, $shownTeachers ) ) continue;
			$teachers[$slug] = str_replace( ' ', '&nbsp;', $teacherObj->post_title );
		}
		return $teachers;
	}
			


	function getClasses() {
		global $wpdb;
		$teachers = $this->getTeachers();
		$classesByDay = array();
		foreach( $this->getDays() as $code => $name )
			$classesByDay[$code] = array();
			
		//$lassesQuery = new WP_Query( "post_type=cours-yoga&post_status=publish&orderby=menu_order&order=ASC&meta_value=$slug_jour" );
		$sqlClasses = "SELECT ID,post_name,post_title FROM {$wpdb->posts} WHERE post_type = 'cours-yoga' AND post_status='publish' ORDER BY menu_order ASC";
		$classResults = $wpdb->get_results( $sqlClasses, OBJECT_K );
		
		foreach( $classResults as $classId => $class ) {
			// un cours = titre, prof, jour, hdeb, hfin
			$teacherSlug = get_post_meta( $classId, 'professeur', true );
			$teacherName = $teachers[$teacherSlug];
			$daySlug = get_post_meta( $classId, 'jour_semaine', true );
			
			$startHour = get_post_meta( $classId, 'heure_debut', true );
			$startHourPretty = WP_Plugin_FormHelper::pretty_print_heure( $startHour );
			$endHour = get_post_meta( $classId, 'heure_fin', true );
			$endHourPretty = WP_Plugin_FormHelper::pretty_print_heure( $endHour );
			
			$model = get_post_meta( $classId, 'modele', true );
			$modelCustom = $model == 'custom' ? get_post_meta( $classId, 'modele_custom', true ) : null;
				
			$classesByDay[$daySlug][$class->post_name] = array( 
				'id'		=> $class->ID, 
				'title'		=> $class->post_title, 
				'teacherSl'	=> $teacherSlug,
				'teacherNm'	=> $teacherName,
//				'day'		=> $days[$daySlug], 
				'stHour'	=> $startHour,
				'endHour'	=> $endHour,
				'stHourPr'	=> $startHourPretty,
				'endHourPr'	=> $endHourPretty,
				'model'		=> $model,
				'modelCust' => $modelCustom
			);
		}
		return $classesByDay;
	}

	public function getClassLevelsBitmask() {
		$bitmaskPosition = 0;
		$classLevelsBitmask = 0;
		$levels = WP_Plugin_FormHelper::wpytt_get_niveaux_kva('');
		$masksPerLevel = array();
		foreach( $levels as $slug => $name ) {
			$thisPositionMask = 1 << $bitmaskPosition;
			//echo "Pos $bitMaskPosition, mask=$thisPositionMask<br>";
			$masksPerLevel[$slug] = $thisPositionMask;
			$classLevelsBitmask += $thisPositionMask;
			$bitmaskPosition++;
			//$table_controls .= "  <td><div class='_$slug tabcontniv'><input class='cbniveau' type='checkbox' id='$slug' name='$slug' value='$mask' checked />$name</div></td>\n";
		}
		$masksPerLevel['__ALL__'] = $classLevelsBitmask;
		return $masksPerLevel;
	}

	public function getGenericBitmaskAndControls( $data, $cbType ) {
		$controls = array();
		$bitmaskPosition = 0;
		$bitmask = 0;
		//$levels = WP_Plugin_FormHelper::wpytt_get_niveaux_kva('');
		$masks = array();
		foreach( $data as $slug => $name ) {
			$thisPositionMask = 1 << $bitmaskPosition;
			//echo "Pos $bitMaskPosition, mask=$thisPositionMask<br>";
			$masks[$slug] = $thisPositionMask;
			$bitmask += $thisPositionMask;
			$bitmaskPosition++;
			$controls[] = "  <span class='_$slug tabcontniv'><span class='checkbox'><input class='$cbType' type='checkbox' id='$slug' name='$slug' value='$thisPositionMask' checked /></span>$name</span>\n";
		}
		$masks['__ALL__'] = $bitmask;
		return array( $masks, $controls );
	}
	
	public function getTermSlugs( $classPostId ) {
		$terms = get_the_terms( $classPostId, 'niveau-cours' );
		if( empty( $terms ) ) return false;
		
		$termSlugs = array();
		foreach( $terms as $term )
			$termSlugs[] = $term->slug;
		return $termSlugs;
	}
	
	public function getTermNiveaux( $classPostId ) {
		$terms_niveaux = get_the_terms( $classPostId, 'niveau-cours' );
		if( empty( $terms_niveaux ) ) return false;

		$slugs_niveaux = array();
		$niveaux = array();
		$niv_matches;
		$nivs_mask = 0;
		$nivs_masks = $this->getClassLevelsBitmask();
		foreach( $terms_niveaux as $term ) {
			$slugs_niveaux[] = $term->slug;
			/*preg_match( '/(\d)+/', $term->name, $niv_matches );
			$niv_number = $niv_matches[0];
			$is_numbered = !empty( $niv_number );
			if( $is_numbered )	$niveaux[] = $niv_number;
			else $niveaux[] = $term->name;*/
			$isNumbered = preg_match( '/(\d)+/', $term->name, $niv_matches ) > 0;
			$niveaux[] = $isNumbered ? $niv_matches[0] : $term->name;
			$nivs_mask += $nivs_masks[$term->slug];
			}
		$slugs_str = implode( ' ', $slugs_niveaux );
		$niveaux_str = ( $isNumbered ? 'N' : '' ) . implode( ',', $niveaux );
		$premier_niv_str = ( $isNumbered ? 'N' : '' ) . $niveaux[0];
		return array( $niveaux, $niveaux_str, $premier_niv_str, $slugs_str, $nivs_mask );
	}
	
	/**
	 * Get empty hour intervals, so that we can show them in a tighter way in the table
	 */
	function computeGaps() {
		
		// We compute the gaps as a "bitmask", i.e. one "bit" = one time unit ($granularity minutes)
		// Get granularity parameter, 
		$granularity = $this->_opts['granularite'];
		if( $granularity == 0 ) $granularity = 1;
		
		$intStartLimit = ( $this->_opts['heure_mini'] | 0 ) / $granularity;
		$intStopLimit = ( $this->_opts['heure_maxi'] | 1440 ) / $granularity;
		$bitsPerHour = 60 / $granularity;
		$loBoundary = $intStopLimit;
		$hiBoundary = $intStartLimit;
		
		/* Clear mask */
		$mask = array();
		for( $i = 0; $i < $intStopLimit - $intStartLimit ; $i++ )
			$mask[$i] = false;

		$classes = $this->getClasses();

		foreach( $classes as $day => $classes )
			foreach( $classes as $slug => $class ) {

				$hourBeginBit = $class['stHour'] / $granularity - $intStartLimit;
				$hourEndBit = $class['endHour'] / $granularity - $intStartLimit;
				for( $index = $hourBeginBit; $index < $hourEndBit; $index++ ) 
					$mask[$index] = true;
			}
		
		/* Get the mask's boundaries */
		for( $i = 0 ; $i < $intStopLimit - $intStartLimit ; $i++ ) {
			if( $mask[$i] && ( $i < $loBoundary ) ) $loBoundary = $i;
			if( $mask[$i] && ( $i > $hiBoundary ) ) $hiBoundary = $i;
			}

		$intervalles = array();
		$i = $loBoundary; $int_deb = 0 ; $int_fin = 0; $num_ints = 0;
		
		while( $i <= $hiBoundary ) {
			if( !$mask[$i] ) {
				
				$int_deb = $i;
				while( !$mask[$i] ) $i++;
				$int_fin = $i;

				$int_deb_mod = $int_deb % $bitsPerHour;
				if( $int_deb_mod > 0 ) $int_deb += ( $bitsPerHour - $int_deb_mod );
				$int_deb /= 4;
				$int_fin_mod = $int_fin % $bitsPerHour;
				if( $int_fin_mod > 0 ) $int_fin -= $int_fin_mod;
				$int_fin /= 4;

				$int_len = $int_fin - $int_deb;
				if( $int_len == 0 ) continue;
				
				$intervalles[] = array( 'debut' => $int_deb, 'fin' => $int_fin, 'duree' => $int_len );
				$num_ints++;
				}
			$i++;
		}

		$total_heures_pause = 0;
		foreach( $intervalles as $intervalle ) {
			$total_heures_pause += $intervalle['duree'];
		}
		
		return array( $intervalles, $total_heures_pause, $mask, $bitsPerHour ); 
	}

}
?>