/**************************************************
 * Class IntervalController = for interval change *
 **************************************************/
var IntervalController = function (model, view) {
    this._model = model;
	this._view = view
	var _this = this;
	var _elems = _this._view._elements;
	
	this._view._elements.modeSelector.change(function (e) { _this.changeSelectionMode(e) });
    this._view._elements.dateSelector.change(function (e) { _this.changeDate(e) });
	
	// register click events, so that the model always know which was the last pressed html element
	for( element in _elems )
		_elems[element].click( function(e) { _this._model.recordAction( e.target.attributes.getNamedItem("name").value ); } );
	

	this._model.intervalChanged.attach(function () { _this.updateSelector(); });

};
 
IntervalController.prototype = {
 
	changeSelectionMode : function(e) {
		this._model.setSelectionMode( e.target.attributes.getNamedItem("value").value );
		return false;
	},
	
	changeDate : function(e) {
		var id = e.target.attributes.getNamedItem("id").value;
		this._model.setCurrentDate( getDateIsoFromRegional( $('#' + id).val() ) );
		return false;
	},

	updateSelector : function () {
		var _this = this;
		_this._view._elements.dateSelector.val( _this._model.getDateIntervalPretty() );
	}

};

/******************************************************
 * Class EntryListController, for :                   *
 *   - for entry table filter & selected entry        *
 *   - add entry button (can deselect selected entry) *
 ******************************************************/
var EntryListController = function ( model, view ) {
    this._model = model;
    this._view = view;
	this._elems = view._elements;
	this._lastClicked = 'null';		/* Last clicked item in the UI	*/
	this._formChanged = false;		/* Has form been changed 		*/
	var _this = this;

	/* Get form state (modified or unchanged from entry data), from the model to which we passed the form data */
	this._model.currentEntryModified.attach( function( model, formChanged ) {
		_this._formChanged = formChanged;
	});
	
	/* Get last clicked element information from model Event notify() */
	this._model.elementClicked.attach( function( model, elemName ) {
//		ddiv( '#debug_actions' , "Element clicked : " + elemName + ", last was " + _this._lastClicked, 'actions' );
//		mclear();
		_this._lastClicked = elemName;
	});

	this._model.listLoaded.attach( function() {
		_this.rebuildList();
	});	
	
};
 
EntryListController.prototype = {
 
	changeFilter : function(e) {
		this._model.setPostFilter( e.target.attributes.getNamedItem("value").value );
		return false;
	},
	
	changeSelectedEntry : function(e) {
		if( ( this._formChanged ) && ( getNameAttr( e ) != this._lastClicked ) )
				mdiv( "L'évènement en cours d'édition a été modifié. Pour abandonner ces modifications, cliquez encore sur cet évènement ("
				+ e.target.innerHTML + ")", 'bgred' );
		else
			this._model.setSelectedEntry( e.target.attributes.getNamedItem("id").value );
		return false;
	},

	rebuildList : function() {
		var _this = this;
		this._view.buildEntryListView( this._elems.entryList, this._model.getEntryList() );
		this._elems.entries = $('.entry');
		this._elems.entries.click(	function(e) { 
			_this.changeSelectedEntry( e ); 
			_this._model.recordAction( e.target.attributes.getNamedItem("name").value );
		} );
//		this._model.setSelectedEntry( -1 );
	}

};


/********************************************************************
 * Class EntryEditController                                        *
 ********************************************************************/
var EntryEditController = function ( model, view ) {
    this._model = model;			/* References to model,			*/
    this._view = view;				/* view, 						*/
	this._elems = view._elements;	/* view's elements				*/
	this._lastClicked = 'null';		/* Last clicked item in the UI	*/
	this._formChanged = false;		/* Has form been changed 		*/
	/* Reference to this instance, for use inside jQuery handlers	*/
	var _this = this;

	/** Register "click" events, so that the model always know which was the last clicked html element
	 *  Warning ! For elements that have to monitor the "last clicked element", we don't bind this default
	 *  handler. Instead we do the same action at the end of each element's click() handler.
	 *  This is to avoid wrong UI behavior (i.e. "a button thinks it's been clicked twice instead of one")
	 *	We also bind "change" events, to send the model the form data for comparison with entry data.
	 */
	this._elems.entryFields.bind( {
		click: function( e ) {
			if( e.target.attributes.getNamedItem("name") != null )
				_this._model.recordAction( e.target.attributes.getNamedItem("name").value ); 
		},
		change: function() {
			_this._model.checkFormAgainstEntry( _this.getFormFields() )
		}
	} );
	
	/* Get form state (modified or unchanged from entry data), from the model to which we passed the form data */
	this._model.currentEntryModified.attach( function( model, formChanged ) {
		_this._formChanged = formChanged;
	});
	
	/* Get last clicked element information from model Event notify() */
	this._model.elementClicked.attach( function( model, elemName ) {
		ddiv( '#debug_actions' , "Element clicked : " + elemName + ", last was " + _this._lastClicked, 'actions' );
		mclear();
		_this._lastClicked = elemName;
	});

	/* Get "selected entry changed" notification, in this case set form from entry data */
	this._model.selectedEntryChanged.attach( function() {
		_this.setViewForm();
		_this._view.triggerChangeEvents();
		//ddiv( '#msgb' , "New entry is selected ? " + _this._model.isSelectedEntryNew() , 'actions' );
		if( _this._model.isSelectedEntryNew() ) _this._elems.newButton.hide();
		else _this._elems.newButton.show();
	});	

	/* Handler for "New" button "click" event */
	this._elems.newButton.click( function( e ) { 
		if( ( _this._formChanged ) && ( getNameAttr( e ) != _this._lastClicked ) )
				mdiv( "L'évènement a été modifié. Pour abandonner ces modifications, cliquez encore sur Nouveau.", 'bgred' );
			else {
				mclear();
				_this._model.createNewEntry();		/* The actual action, i.e. set form fields from model	*/
				_this._view.triggerChangeEvents();
			}
		_this._model.recordAction( getNameAttr( e ) ); 
	} );


	/** Action for discard button click, can be :
	 *	1- If form has been changed AND last element clicked is different from discard button, show warning
	 *	2- If warning's been ignored by clicking a 2nd time, do action, remove warning, maybe (?) reset last clicked
	 */
	this._elems.discardButton.click( function( e ) { 
		if( _this._formChanged ) {
			if( getNameAttr( e ) != _this._lastClicked )
				mdiv( "L'évènement a été modifié. Pour abandonner ces modifications, cliquez encore sur Annuler.", 'bgred' );
			else {
				mclear();
				_this.setViewForm();			/* The actual action, i.e. set form fields from model	*/
				_this._view.triggerChangeEvents();
			}
		}
		_this._model.recordAction( getNameAttr( e ) ); 
	} );

	// action for save button click
	this._elems.saveButton.click( function( e ) { 
		_this.submitForm( e ); 
		_this._model.recordAction( getNameAttr( e ) ); 
	} );
	
};
 
EntryEditController.prototype = {

	getFormFields : function () {
		var entry = this._model.getSelectedEntry();

		var str = ''; var i = 0;
		var formFields = {};
		var actions = { 
			'text' : function( elem ) { return elem.value; },
			'select' : function( elem ) { return $( 'select[name=' + elem.name + '] option:selected' ).val(); },
			'checkbox' : function( elem ) { return $(elem).is(':checked'); },
			'radio' : function( elem ) {
				var value = null;
				$( 'input[name=' + elem.name + ']' ).each( function() {
					if( $(this).is(':checked') ) value = this.value; } );
				return value;
			}
		}
		this._view._elements.entryFields.each( function(  ) { 
			if( ( !( $( this ).is(":visible") ) && ( this.type != 'hidden' ) ) || ( $( this ).is(":disabled") ) ) return true;
			var nodeName = this.nodeName.toLowerCase();
			var fieldType = this.type;
			var fieldValue = this.value;
			var type = 'text';
			if( nodeName == 'select' ) type = 'select';
			if( ( nodeName == 'input' ) && ( fieldType == 'radio' || fieldType == 'checkbox' ) )
				type = fieldType;
			
			if( formFields[this.name] == undefined ) {
				formFields[this.name] = ( actions[type] != undefined ) ? actions[type](this) : 'NOACT';
				str += this.name + '=' + formFields[this.name] + '<br>';
			str +=  i + '- Node "' + nodeName + '" with type "'+ fieldType + '" and name "'
				+ this.name + '" is assigned type ' + type + '<br>'
				+ '\t action returned : ' + ( ( actions[type] != undefined ) ? this.name + '=' +actions[type](this) : 'NOACT' ) + '<br>';
			i++;
			}

		});
		$( "#debug_conts" ).html( str );
		return formFields;
	
	},
	
	submitForm : function( event ) {
		// cas ou aucune entree n'est sélectionnée : à l'init, ou si on crée un nouvel event

		invalidFields = entry.validateAndSave( this.getFormFields() );
		if( invalidFields.length > 0 )
			this._view.showInvalidFields( invalidFields );
		else {
			this._model.writeBackEntry();
			this._model.checkFormAgainstEntry( this.getFormFields() )
			mdiv( 'Evènement sauvegardé.', 'bggreen' );
		}
	},
	
	setViewForm: function() { 
		this._view.fillEntryFields( this._model.getSelectedEntry().getAll() );
		kvdiv( 'views', this._model.getSelectedEntry().getAll() );
		//dbg( this._model.getSelectedEntry().getAll() );

	}


};