/** For Class inheritance */
function setPropertiesFrom( target, source ) {
	for( k in source ) target[k] = source[k];
}


/******************************************************
 * CalendarEntryModel class, model for calendar entry *
 ******************************************************/
var CalendarEntryModel = function( entry, fields ) {

	// Fields to be mapped to HTML form fields : default values
	this.fields = {
		'title': '',			'desc': '',			'where': '',		'color': '',
		'startdate': '',		'starttime': '',	'enddate': '',		'endtime': '',		'wholeday': false,		'_allTimes':'',
		'recurrent': false,		'FREQ': '',			'INTERVAL': '',
		'MO': false,			'TU': false,		'WE': false,		'TH': false,
		'FR': false,			'SA': false,		'SU': false,
		'repeatmonthtype': '',	'BYDAY': '', 		'BYMONTHDAY': '',	// BYMONTHDAY | BYDAY : nth of month, nth Mo/Su of month
		'stopwhentype': '',		'COUNT': '',		'UNTIL': ''			// _NEVER_, COUNT, UNTIL
	}
	
	// Initialize entry and fields
	if( entry != null )
		this.entry = entry;
	else this.entry = null;
	if( fields != null )
		this.setFields( fields );
	//dbg( fields._allTimes );
	
}

CalendarEntryModel.prototype = {

	getEntry : function() {
		return this.entry;
	},
	
	setEntry : function( entry ) {
		this.entry = entry;
	},
	
	/*** Getter and setter functions for fields ***/
	
	get : function( key ) {
		return this.fields[key];
	},
	getAll : function() {
		return this.fields;
	},
	set : function( key, value ) {
		this.fields[key] = value;
	},
	setFields : function( kva ) {
		for( k in kva ) this.fields[k] = kva[k];
	},

	validateAndSave : function( data ) {
		var metadata = {
			'title' : { 'required': true, 'type': 'text' },
			'startdate' : { 'required': true, 'type': 'date' },
			'enddate' : { 'required': true, 'type': 'date', 'min': 'startdate' },
			'starttime' : { 'type': 'time' },
			'endtime': {'type': 'time', 'min': 'starttime' },
			'repeatmonthtype': { 'type': 'text' },
			'BYDAY': { 'required_if_parent': { field: 'repeatmonthtype', value: 'BYDAY' }, 'type': 'date' },
			'BYMONTHDAY': { 'required_if_parent': { field: 'repeatmonthtype', value: 'BYMONTHDAY' }, 'type': 'number' },
			'stopwhentype': { 'type': 'text' },
			'UNTIL': { 'required_if_parent': { field: 'stopwhentype', value: 'UNTIL' }, 'type': 'date' },
			'COUNT': { 'required_if_parent': { field: 'stopwhentype', value: 'COUNT' }, 'type': 'number' }
		};
		var patterns = {
			'date' : '^[0-9]{1,2}/[0-9]{1,2}/[0-9]{4}$',
			'time' : '^[0-9]{1,2}:[0-9]{2}$',
			'number' : '^[0-9]+$'
		};
		var isNull = {
			'text' : function( text ) { return ( text.length == 0 ); },
			'time' : function( text ) { return ( text.length == 0 ); },
			'date' : function( text ) { return ( text.length == 0 ); },
			'number' : function( number ) { return ( ( '' + number ).length == 0 ); },
		};
	/*	Ajouter opérateurs de comparaison de DateTime !
		var operators = {
			'date' : function( d1, d2 ) {
				dR1 = getDateTimeRegionalFromGdataWhen( d1 );
				dJs1 = getDateTimeJsFromRegional( dR1 );
				dR2 = getDateTimeRegionalFromGdataWhen( d2 );
				dJs1 = getDateTimeJsFromRegional( dR1 );
	*/			
		var invalidFields = [];
		
		// boucle sur tableau key-value a valider
		for (var key in data ) {
			// si metadata non-definie, passe a cle suivante
			if( metadata[key] == undefined ) continue;
			// recupere la valeur
			value = data[key];
			// erreur si : valeur requise et absente, ou valeur typee et mal formattee
			if( metadata[key].required != undefined && isNull[metadata[key].type]( value ) )
				invalidFields[key] = {
					error: 'MISSING',
					message: 'Le champ ' + key + ' est obligatoire\n'
				}
			if(	metadata[key].required_if_parent != undefined
				&& data[metadata[key].required_if_parent.field] == metadata[key].required_if_parent.value
				&& isNull[metadata[key].type]( value ) )
					invalidFields[key] = {
						error: 'MISSING',
						message: 'Le champ ' + key + ' est obligatoire\n'
					}
			if( metadata[key].type != undefined && !isNull[metadata[key].type]( value ) )
				if( patterns[metadata[key].type] != undefined ) {
					re = new RegExp( patterns[metadata[key].type] );
					if( !re.test( value ) )
					invalidFields[key] = {
						error: 'BADFORMAT',
						message: 'Le champ ' + key + ' est mal formaté\n'
					}
				}
		}
		// everything ok, we set the selected entry's fields with provided data
		if( invalidFields.length == 0 ) this.setFields( data );
		// whether we have invalid fields or not, return this array to controller
		return invalidFields;
	}

};


CalendarEntryModel.recurrenceBits = jQuery.parseJSON ( 
'{ "recurrences": ['
	+ '{"index": "0", "key": "DTSTART;TZID=Europe/Paris", "val": "20110913T121500"},'
	+ '{"index": "1", "key": "DTEND;TZID=Europe/Paris", "val": "20110913T131500"},'
	+ '{"index": "2", "key": "RRULE", "val": "FREQ=WEEKLY;UNTIL=20120626T101500Z;BYDAY=TU"},'
	+ '{"index": "3", "key": "BEGIN", "val": "VTIMEZONE"},'
	+ '{"index": "4", "key": "TZID", "val": "Europe/Paris"},'
	+ '{"index": "5", "key": "X-LIC-LOCATION", "val": "Europe/Paris"},'
	+ '{"index": "6", "key": "BEGIN", "val": "DAYLIGHT"},'
	+ '{"index": "7", "key": "TZOFFSETFROM", "val": "+0100"},'
	+ '{"index": "8", "key": "TZOFFSETTO", "val": "+0200"},'
	+ '{"index": "9", "key": "TZNAME", "val": "CEST"},'
	+ '{"index": "10", "key": "DTSTART", "val": "19700329T020000"},'
	+ '{"index": "11", "key": "RRULE", "val": "FREQ=YEARLY;BYMONTH=3;BYDAY=-1SU"},'
	+ '{"index": "12", "key": "END", "val": "DAYLIGHT"},'
	+ '{"index": "13", "key": "BEGIN", "val": "STANDARD"},'
	+ '{"index": "14", "key": "TZOFFSETFROM", "val": "+0200"},'
	+ '{"index": "15", "key": "TZOFFSETTO", "val": "+0100"},'
	+ '{"index": "16", "key": "TZNAME", "val": "CET"},'
	+ '{"index": "17", "key": "DTSTART", "val": "19701025T030000"},'
	+ '{"index": "18", "key": "RRULE", "val": "FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU"},'
	+ '{"index": "19", "key": "END", "val": "STANDARD"},'
	+ '{"index": "20", "key": "END", "val": "VTIMEZONE"}'
	+ ']}'
);




/******************************************************
 * CalendarModel class, this is our model super-class *
 ******************************************************/
var CalendarModel = function() {
	
	this.selectionMode = null;
	this.currentDate = null;
	this.dateInterval = null;
	this.calendarEntries = null;
	this.newEntry = null;
	this.selectedEntryIndex = -1;
	this.postFilter = null;
	this.filterShowAll = 'Tous';
	this.listReady = false;
	this.recParser = new RecurrenceParser( );
	
	this.intervalChanged	  =	new Event( this );
	this.filterChanged	 	  = new Event( this );
	this.currentEntryModified = new Event( this );
	this.elementClicked		  = new Event( this );
	this.selectedEntryChanged = new Event( this );
	this.listLoaded			  = new Event( this );
	
};


/***************************
 * CalendarModel prototype *
 ***************************/
CalendarModel.prototype = {

	/*hasSelectedEntry : function() {
		return( this.selectedEntry != -1 );
	}*/
	
/*	isListReady : function() {
		return this.listReady;
	},
	*/
	
	
	/*** INITIALIZATION ***/
	init : function() {
		this.selectionMode = 'week';
		this.currentDate = new DateModel( new Date() );
		this.setDateInterval( this.currentDate.getWeekIntervalGC() );
		this.intervalChanged.notify();
		this.fetchEvents();
		this.createNewEntry();
	},
	

	/*** DATE, SELECTION MODE, FILTER METHODS ***/

	getDateInterval : function() {
		return this.dateInterval;
	},
	
	getDateIntervalPretty : function() {
		switch( this.selectionMode ) {
			case 'month' : return this.currentDate.getMonthAndYear();
			case 'week' : return this.currentDate.getWeekIntervalPretty();
		}
	},

	setCurrentDate : function( dateIso ) {
		this.currentDate.setDateIso ( dateIso );
		this.resetDateInterval();
		this.intervalChanged.notify();
	},

	setDateInterval : function( dateArray ) {
		this.dateInterval = dateArray;
	},

	resetDateInterval : function() {
		if( this.selectionMode == 'week' ) this.setDateInterval( this.currentDate.getWeekIntervalGC() );
		else if( this.selectionMode == 'month' ) this.setDateInterval( this.currentDate.getMonthInterval() );
		this.fetchEvents();
	},

	setSelectionMode : function( mode ) {	// selection Mode (day, week, month)
		this.selectionMode = mode;
		this.resetDateInterval();
		this.intervalChanged.notify();
	},
	
	setPostFilter : function( filter ) {
		this.postFilter = ( filter != filterShowAll ) ? filter : null;
		this.filterChanged.notify();
	},
	
	/*** ENTRY LIST METHODS ***/

	/** Get entries */
	getEntryList : function() {
		
		var entryList = new Array();
		
		for (var i = 0; i < this.calendarEntries.length; i++) {
			if( this.calendarEntries[i].get( "title" ) == this.postFilter ) continue;
			entryList.push( this.calendarEntries[i] );
		}
		
		return entryList;
	},
	
	getEntryNumber : function() {
		return this.calendarEntries.length;
	},

	/*** SINGLE ENTRY METHODS ***/
	
	/** L'entree selectionne est soit la nouvelle entrée, soit une de la liste */
	getSelectedEntry : function() {
		if( this.newEntry != null )						// d'abord la nouvelle entree
			return this.newEntry;
		else if( this.selectedEntryIndex >= 0 )			// ou une entrée du tableau
			return this.calendarEntries[this.selectedEntryIndex];
		$('debug_models').html('ERR CRITICAL');
		return null;									// ne devrait jamais arriver
	},
	
	/** Est-ce la nouvelle entrée qui est sélectionnée ? */
	isSelectedEntryNew : function() {
		return( this.newEntry != null );
	},

	setSelectedEntry : function( index ) {
		this.selectedEntryIndex = index;
		if( index != -1 ) {
			delete this.newEntry;
			this.newEntry = null;
		}
		this.selectedEntryChanged.notify();
	},

	createNewEntry : function() {
		this.newEntry = new CalendarEntryModel( null, null );
		this.setSelectedEntry( -1 );
	},
	
	/** Loop through all form's set fields, and fire "currentEntryModified" = true on the first difference found with model.
	 *	On the contrary, fires "currentEntryModified" = false if no difference is found */
	checkFormAgainstEntry : function( formFields ) {
		entry = this.getSelectedEntry();
		for( field in formFields ) if( formFields[field] != entry.get( field ) ) {
			str += 'Champ "' + field + '" modifié: new='+formFields[field] + ', old=' + entry.get(field) + '<br>';
			ddiv( '#debug_models', str, 'models', false, true );
			this.currentEntryModified.notify( true );
			return;
		}
		this.currentEntryModified.notify( false );
	},
	
	recordAction : function( elemName ) {
		this.elementClicked.notify( elemName );
	},
	
	
	//deleteEntry: function( 

	/** Utility function to format numbers so that single digits have a 0 in front. */
		formatNumber : function(num) {
	  if (num.length == 1) {
		return '0' + num;
	  }
	  return num;
	}

};





/***********************************************
 * CalendarModelGoogle, subclass of our model  *
 ***********************************************/
CalendarModelGoogle.prototype = new CalendarModel;
CalendarModelGoogle.prototype.constructor = CalendarModelGoogle;

function CalendarModelGoogle() {

	/*** Members ***/

	//this.eventFeedUrl = feedUrl;
	//this.eventFeedUrl = 'http://www.google.com/calendar/feeds/asso.aray@gmail.com/public/full'; // url for user's default calendar
	CalendarModel.call( this );
	this.eventFeedUrl = 'http://www.google.com/calendar/feeds/benoithubert@gmail.com/public/full'; // url for user's default calendar
	this.calendarService = null; // used for retrieving and adding events
	var _this = this;

	/**
	 * NOT IN PROTOTYPE, otherwise in the function can't access "this"
	 * Called after events query returns: build entry list.
	 * @param {google.gdata.CalendarFeed} resultsFeedRoot The events feed
	 */
	this.handleEventsFeed = function (resultsFeedRoot) {
		eventsFeed = resultsFeedRoot.feed;
		//_this.insertIntoMyFeed( resultsFeedRoot );
		events = eventsFeed.getEntries();
		
		_this.calendarEntries = new Array( events.length );

		for( var i = 0 ; i < events.length ; i++ )
			_this.calendarEntries[i] = new CalendarEntryModel( events[i], _this.mapToEntryFields( events[i] ) );
			
		//_this.buildEntryList( tempEntries );
		_this.listReady = true;
		_this.listLoaded.notify();
	}
	
	/**
	 * Build Entry list from a list of submodel-specific events
	 */
/*	this.buildEntryList = function( events ) {
		
		this.calendarEntries = new Array( events.length );
		
		for (var i = 0; i < this.calendarEntries.length; i++) {
			this.calendarEntries[i] = new CalendarEntryModel( i, events[i] );
		}
		
	}*/

};

setPropertiesFrom( CalendarModelGoogle.prototype, {

	/** INITIALIZATION */
	
	init : function() {
		this.calendarService = new google.gdata.calendar.CalendarService( 'calendarEntriesManager' );
		CalendarModel.prototype.init.call( this );
	},

	/** 
	 * Return human-readable values
	 */
	toString : function() {
		return	'[ calendarService = '	+ this.calendarService + 
				', selectionMode = '	+ this.selectionMode + 
				', currentDate = '		+ this.currentDate + 
				', dateInterval = '		+ this.dateInterval + ' ]';
	},
	
	/**
	 * This function is triggered by the login/logout button.
	 * If the user is logged in to the app, it logs them out.
	 * If the user is logged out to the app, it logs them in.
	 */
	loginOrLogout : function() {
		  var token = google.accounts.user.checkLogin( this.eventFeedUrl );
		  if (token) {
			google.accounts.user.logout();
			this.init();
		  } else {
			google.accounts.user.login(this.eventFeedUrl);
		  }
	},
	
	/**
	 * Retrieves birthday reminders in the next 2 months,
	 * and calls handleEventsFeed for the results.
	 */
	fetchEvents : function () {
		var eventQuery = new google.gdata.calendar.CalendarEventQuery( this.eventFeedUrl );

		eventQuery.setSortOrder('ascending');
		eventQuery.setMaxResults( 1000 );
		eventQuery.setSingleEvents( false );
		eventQuery.setMinimumStartTime( this.dateInterval[0] );
		eventQuery.setMaximumStartTime( this.dateInterval[1] );
		this.listReady = false;
		this.calendarService.getEventsFeed(eventQuery, this.handleEventsFeed, this.handleError);
	},

	/**
	 *	writes a generic entry to the specific data support
	 */
	 writeBackEntry : function() {
		entry = this.getSelectedEntry();
		this.mapBackAndWrite( entry );
	 },

	 /** Maps the Gdata CalendarEventEntry properties to CalendarEntryModel fields */
	mapToEntryFields : function( entry ) {

		var fields = {};
		fields["title"]	= entry.getTitle().getText();
		fields["desc"]	= entry.getContent().getText();
		fields["where"]	= 'UNSUPPORTED';
		
		var times = entry.getTimes();
		var timeList = new Array( times.length );
		for( i = 0 ; i < times.length ; i++ )
			timeList[i] = { 'startTime'	: getDateTimeRegionalFromGdataWhen( times[i].startTime ),
							'endTime'	: getDateTimeRegionalFromGdataWhen( times[i].endTime ) };
		fields["_allTimes"]	= timeList;

		//var locations = entry.getLocations();
		//this.set( "where", ( locations != undefined ) ? locations[0].getValue() : '');
		
		if( entry.getRecurrence() != undefined ) {
			var variables = this.recParser.parse( '^[A-Z]*.*', entry.getRecurrence().getValue() );
			//for( k in variables ) alert( 'key: ' + k + ', value:' + variables[k] );
			setPropertiesFrom( fields, variables );
		}
		else {
			dateTimeStart = timeList[0].startTime;
			dateTimeEnd = timeList[0].endTime;
			fields["startdate"]	= dateTimeStart[0];
			fields["starttime"]	= dateTimeStart[1];
			fields["enddate"]	= dateTimeEnd[0];
			fields["endtime"]	= dateTimeEnd[1];

		}
		var f = fields['FREQ'];
		if( ( f == 'DAILY' || f == 'WEEKLY' || f == 'MONTHLY' || f == 'YEARLY' ) && fields['INTERVAL'] == undefined )
			fields['INTERVAL'] = '1';

		if( fields["starttime"] == '' ) fields["wholeday"] = true;
		str = "<table><h3>Mappage des champs GC=>CalendarEntryModel</h3>";
		//entry = this.getSelectedEntry();
		kvdiv('#debug_models', fields, 'models');
		//$('#debug_models').html( 'Select entree "' + this.getSelectedEntry().get( "title" ) );
		
		return fields;
	
	},
	
	/**
	 * Utility function for converting extended properties array
	 *  into a hash function so it's easy to access named properties.
	 *  @param {Array} extPropertiesArray Array of extended properties
	 *  @return {Object} Hash of extended properties
	 */
	getExtPropertiesHash : function(extPropertiesArray) {
	  var extPropertiesHash = {};
	  //dbg = 'Properties ? \n';
	  for (var i = 0; i < extPropertiesArray.length; i++) {
	   var extProperty = extPropertiesArray[i];
	   extPropertiesHash[extProperty.name] =extProperty.value;
	   //dbg += 'name: [' + extProperty.name + '], value: [' + extProperty.value + ']\n';
	  }
	  //alert( dbg );
	  return extPropertiesHash;
	},

	insertIntoMyFeed : function(feedRoot) {
		  var newEntry = new google.gdata.calendar.CalendarEventEntry({
			  authors: [{
				name: "Benoît Hubert",
				email: "benoithubert@gmail.com"
			  }],
			  title: {
				type: 'text', 
				text: 'Me prendre une grosse caisse pour fêter tout ça!'
			  },
			  content: {
				type: 'text', 
				text: 'Prévoir un pack de 26, une bouteille de gros-plant, par personne bien sûr'
			  },
			  locations: [{
				rel: "g.event",
				label: "Event location",
				valueString: "A la maison, pour changer !"
			  }],
			  times: [{
				startTime: google.gdata.DateTime.fromIso8601("2011-11-02T18:00:00.000Z"),
				endTime: google.gdata.DateTime.fromIso8601("2011-11-03T05:00:00.000Z")
			  }]
				
		  });
		//	newEntry.setRecurrence( "DTSTART;VALUE=DATE:20100505\r\nDTEND;VALUE=DATE:20100506\r\nRRULE:FREQ=WEEKLY;BYDAY=Tu;UNTIL=20111004\r\n" );
			  this.calendarService.insertEntry( this.eventFeedUrl, newEntry, this.entryCreated, this.handleError);
		//  feedRoot.feed.insertEntry(newEntry, null, null);
	},	
	
	/** Maps the Gdata CalendarEventEntry properties to CalendarEntryModel fields */
	mapBackAndWrite : function( genEntry ) {

		var event;
		if( genEntry.getEntry() == null )
			genEntry.setEntry( new google.gdata.calendar.CalendarEventEntry() );
		event = genEntry.getEntry();
		fields = genEntry.getAll();
		
		event.setTitle( google.gdata.atom.Text.create( '*' + fields["title"] + '*' ) );
		event.setContent( google.gdata.atom.Text.create( fields["desc"] ) );
		
		if( fields["recurrent"] ) 
			var rRule = this.recParser.build( fields );
		
		//ddiv( '#debug_models', rRule + '<br>', 'models' );
		//kvdiv( 'models', fields );
		
		/*event.updateEntry(
			function(result) {
				mdiv('event updated!', 'bggreen' );
			},
			this.handleError
		);*/
		ddiv( '#msgc', 'Credentials: <br>' + flatkv( this.calendarService.getUserCredentials() ), 'bgblue' );
		ddiv( '#debug_models', event.getTitle().getText(), 'notifs' );
		ddiv( '#debug_models', flatkv( fields ), 'notifs', false );
		ddiv( '#debug_models', flatkv( event ), 'notifs', false );
		newEvent = new google.gdata.calendar.CalendarEventEntry();
		newEvent.setTitle( event.getTitle() );
		
		myOkHandler = function( eRoot ) { ddiv( '#messages', 'Goooooooooooaaaaaaaaaaaaallllllll!', 'bggreen', false ); };
		myErrHandler = function( e ) { ddiv( '#messages', e.cause ? e.cause.statusText : e.message, 'bgred', false ); };
//		this.calendarService.insertEntry( this.eventFeedUrl, newEvent, this.entryCreated, this.handleError);
		this.calendarService.updateEntry( this.eventFeedUrl, event, myOkHandler, myErrHandler );
//		this.calendarService.updateEntry( event.getId().getValue(), event, myOkHandler, myErrHandler );
//		event.deleteEntry( myOkHandler, myErrHandler );
//		this.calendarService.insertEntry( this.eventFeedUrl, newEvent, this.entryCreated, this.handleError);
		/*
		//fields["where"]	= 'UNSUPPORTED';
		
		var times = entry.getTimes();
		var timeList = new Array( times.length );
		for( i = 0 ; i < times.length ; i++ )
			timeList[i] = { 'startTime'	: getDateTimeRegionalFromGdataWhen( times[i].startTime ),
							'endTime'	: getDateTimeRegionalFromGdataWhen( times[i].endTime ) };
		fields["_allTimes"]	= timeList;

		//var locations = entry.getLocations();
		//this.set( "where", ( locations != undefined ) ? locations[0].getValue() : '');
		
		if( entry.getRecurrence() != undefined ) {
			var variables = recParser.parse( '^[A-Z]*.*', entry.getRecurrence().getValue() );
			//for( k in variables ) alert( 'key: ' + k + ', value:' + variables[k] );
			setPropertiesFrom( fields, variables );
		}
		else {
			dateTimeStart = timeList[0].startTime;
			dateTimeEnd = timeList[0].endTime;
			fields["startdate"]	= dateTimeStart[0];
			fields["starttime"]	= dateTimeStart[1];
			fields["enddate"]	= dateTimeEnd[0];
			fields["endtime"]	= dateTimeEnd[1];

		}
		var f = fields['FREQ'];
		if( ( f == 'DAILY' || f == 'WEEKLY' || f == 'MONTHLY' || f == 'YEARLY' ) && fields['INTERVAL'] == undefined )
			fields['INTERVAL'] = '1';

		if( fields["starttime"] == '' ) fields["wholeday"] = true;
		str = "<table><h3>Mappage des champs GC=>CalendarEntryModel</h3>";
		//entry = this.getSelectedEntry();
		kvdiv('#debug_models', fields, 'models');
		//$('#debug_models').html( 'Select entree "' + this.getSelectedEntry().get( "title" ) );
		
		return fields;
	*/
	},
	
	/**
	 * Adds birthday reminder event to calendar, based on form.
	 */
		saveEvent : function() {
	  // Get values from form
	  var birthdateINPUT = document.getElementById('startdate').value;
	  var birthdateInputSplit = birthdateINPUT.split('/');
	  if (birthdateInputSplit.length < 3) {
		alert('Birthdate should be in M/D/YYYY format, e.g. 6/24/1984.');
		return;
	  }
	  var birthday = birthdateInputSplit[1];
	  var birthmonth = birthdateInputSplit[0];
	  var birthyear = birthdateInputSplit[2];
	  var name = document.getElementById('name').value;
	  var image = document.getElementById('image').value;
	  var profile = document.getElementById('profile').value;
	  var inputs = { 'name': name, 'birthmonth': birthmonth, 
		  'birthday': birthday, 'birthyear': birthyear }; //, 'image': image, 'profile': profile};

	  // Create event
	  var event = new google.gdata.calendar.CalendarEventEntry();

	  // Add title - this will be shown in agenda view, so it should be useful
	  event.setTitle(google.gdata.Text.create('Birthday Reminder for ' + name));

	  // Add recurrence - birthday reminders should be repeated annually
	  var today = new Date();
	  var formattedStart =  today.getFullYear() + '' + formatNumber(birthmonth) + 
		  '' + formatNumber(birthday);
	  var recurrence = new google.gdata.Recurrence();

	  // The recurrence value is in ICAL format. More info available here:
	  // http://code.google.com/apis/calendar/developers_guide_protocol.html#CreatingRecurring
	  recurrence.setValue('DTSTART;VALUE=DATE:' + formattedStart
		   + 'DTEND;VALUE=DATE:' + formattedStart
		   + '\r\nRRULE:FREQ=YEARLY;WKST=SU');
	  event.setRecurrence(recurrence);

	  // Add reminders - we have 3 checkboxes in the form.
	  // Each number represents number of days. 
	  // To add different options, change this array and checkbox HTML.
	  // Options need to match options available in the calendar UI.
	  var reminderDays = ['1', '2', '7'];

	  for (var i = 0; i < reminderDays.length; i++) {
		if (document.getElementById('reminder_days' + reminderDays[i]).checked) {
		  event.addReminder({days: reminderDays[i], method: google.gdata.Reminder.METHOD_EMAIL});
		}
	  }

	  // Add (empty) content
	  event.setContent(google.gdata.Text.create(''));

	  // Add web content - this will place an icon on the calendar
	  //  that opens a calendar gadget when clicked. 
	  //  We set user preferences so the gadget knows the 
	  //  name, birthday, and image to display.
	  var webContent = new google.gdata.calendar.WebContent();
	  webContent.setHeight(240);
	  webContent.setWidth(280);
	  webContent.setUrl('http://gdata-javascript-client.googlecode.com/svn/trunk/samples/calendar/birthday_manager/birthday_manager_gadget.xml');
	  for (input in inputs) {
		var gadgetPref = new google.gdata.calendar.WebContentGadgetPref();
		gadgetPref.setName(input);
		gadgetPref.setValue(inputs[input]);
		webContent.addGadgetPref(gadgetPref);
	  }

	  var calendarLink = new google.gdata.calendar.CalendarLink();
	  calendarLink.setWebContent(webContent);
	  calendarLink.setRel('http://schemas.google.com/gCal/2005/webContent');
	  calendarLink.setHref(
		  'http://gdata-javascript-client.googlecode.com/svn/trunk/samples/calendar/birthday_manager/images/fingerreminder_favicon.png');
	  calendarLink.setTitle('Birthday Reminder for ' + name);
	  calendarLink.setType('application/x-google-gadgets+xml');
	  
	  event.setWebContentLink(calendarLink);
	  
	  // Add extended properties - This is a structured way 
	  //  of storing additional information in a calendar entry. 
	  //  They won't show up in the UI, but it will show up in the feed.
	  for (input in inputs) {
		var extendedProperty = new google.gdata.ExtendedProperty();
		extendedProperty.setName(input);
		extendedProperty.setValue(inputs[input]);
		event.addExtendedProperty(extendedProperty);
	  }

	  // Insert entry into the default calendar for this service
	  this.calendarService.insertEntry( this.eventFeedUrl, event, entryCreated, handleError);
	},

	/**
	 * Called when event is added.
	 * It updates the upcoming birthdays table.
	 * @param {google.gdata.calendar.CalendarEntry} entryRoot
	 */
	entryCreated : function(entryRoot) {
	  mdiv( 'Successfully added to calendar.', 'bggreen' );
	  //this.getEvents();
	},


	/** Called if an error is encountered while retrieving a feed or adding an event. */
	handleError : function(e) {
		mdiv( e.cause ? e.cause.statusText : e.message, 'bgred' );
		ddiv( '#debug_views', '<b>Status</b><br>' + flatkv( e.cause ) + '<br><hr><br>' + flatkv( e.cause.responseHeaders ), 'bgred', false );
		//kvdiv( 'views', e );
	}

}
);


















/***********************************************
 * CalendarModelWP, subclass of our model  *
 ***********************************************/
CalendarModelWP.prototype = new CalendarModel;
CalendarModelWP.prototype.constructor = CalendarModelWP;

function CalendarModelWP() {

	/*** Members ***/

	CalendarModel.call( this );
	this.calendarSlug = 'default';
	this.wpOk;
	var _this = this;
	mdiv( 'constructor CalendarModelWP', 'bggreen' );
	
	this.ajaxHandler = function( json ){
		
		data = $.parseJSON( json );
		//_this.wpOk = ( data.length > 0 );
		//alert( serialize( data ) );
		//mdiv( data[6].id, 'bgblue' );
		ddiv( '#debug_actions', flatkvs( data ), 'actions' );
	}

};

setPropertiesFrom( CalendarModelWP.prototype, {

	/** INITIALIZATION */
	
	init : function() {
		CalendarModel.prototype.init.call( this );
		//this.wpOK = this.queryWP( { 'query_type' : 'init' } );
		this.fetchEvents();
		if( !this.wpOk ) mdiv( 'Erreur initialisation module comm WP', 'bgred' );
		else  mdiv( 'Initialisation module comm WP OK', 'bggreen' );
		//mdiv( 'init CalendarModelWP', 'views' );
	},

	/** 
	 * Return human-readable values
	 */
	toString : function() {
		return	'[ selectionMode = '	+ this.selectionMode + 
				', currentDate = '		+ this.currentDate + 
				', dateInterval = '		+ this.dateInterval + ' ]';
	},
	
	/** This function is triggered by the login/logout button. */
	loginOrLogout : function() {
		//  var token = wp.checkLogin();
	},
	
	/** Retrieves events. */
	fetchEvents : function () {
		
		queryParams = {};
		queryParams['query_type'] = 'read';
		queryParams['post_type'] = 'page';
		queryParams['order'] = 'ASC';
		queryParams['orderby'] = 'post_title';
		queryParams['posts_per_page']	 = -1 ;
		queryParams['dtMin'] = this.dateInterval[0];
		queryParams['dtMax'] = this.dateInterval[1];
		ddiv( '#msgb', flatkv( queryParams ), 'actions' );
		
		this.listReady = false;

		this.calendarEntries = this.queryWP( queryParams );
//			_this.calendarEntries[i] = new CalendarEntryModel( events[i], _this.mapToEntryFields( events[i] ) );
			
		this.listReady = true;
		this.listLoaded.notify();
	},
	
	queryWP : function( params ) {
		ddiv( '#msgc', $('#_ajaxUrl').val(), 'bgred' );
		url = $('#_ajaxUrl').val();
		$.ajax({
			url: url,
			data: params,
			dataType: "text",
			//context: document.body,
			success: this.ajaxHandler
		});
		
	},

	/**
	 *	writes a generic entry to the specific data support
	 */
	 writeBackEntry : function() {
		entry = this.getSelectedEntry();
		this.mapBackAndWrite( entry );
	 },

	 /** Maps the Gdata CalendarEventEntry properties to CalendarEntryModel fields */
	mapToEntryFields : function( entry ) {

		var fields = {};
		fields["title"]	= entry.getTitle().getText();
		fields["desc"]	= entry.getContent().getText();
		fields["where"]	= 'UNSUPPORTED';
		
		var times = entry.getTimes();
		var timeList = new Array( times.length );
		for( i = 0 ; i < times.length ; i++ )
			timeList[i] = { 'startTime'	: getDateTimeRegionalFromGdataWhen( times[i].startTime ),
							'endTime'	: getDateTimeRegionalFromGdataWhen( times[i].endTime ) };
		fields["_allTimes"]	= timeList;

		//var locations = entry.getLocations();
		//this.set( "where", ( locations != undefined ) ? locations[0].getValue() : '');
		
		if( entry.getRecurrence() != undefined ) {
			var variables = this.recParser.parse( '^[A-Z]*.*', entry.getRecurrence().getValue() );
			//for( k in variables ) alert( 'key: ' + k + ', value:' + variables[k] );
			setPropertiesFrom( fields, variables );
		}
		else {
			dateTimeStart = timeList[0].startTime;
			dateTimeEnd = timeList[0].endTime;
			fields["startdate"]	= dateTimeStart[0];
			fields["starttime"]	= dateTimeStart[1];
			fields["enddate"]	= dateTimeEnd[0];
			fields["endtime"]	= dateTimeEnd[1];

		}
		var f = fields['FREQ'];
		if( ( f == 'DAILY' || f == 'WEEKLY' || f == 'MONTHLY' || f == 'YEARLY' ) && fields['INTERVAL'] == undefined )
			fields['INTERVAL'] = '1';

		if( fields["starttime"] == '' ) fields["wholeday"] = true;
		str = "<table><h3>Mappage des champs CalWP => CalendarEntryModel</h3>";
		//entry = this.getSelectedEntry();
		kvdiv('#debug_models', fields, 'models');
		//$('#debug_models').html( 'Select entree "' + this.getSelectedEntry().get( "title" ) );
		
		return fields;
	
	},
	
	/** Maps the Gdata CalendarEventEntry properties to CalendarEntryModel fields */
	mapBackAndWrite : function( genEntry ) {

		var event;
		if( genEntry.getEntry() == null )
			genEntry.setEntry( new wp.CalendarEventEntry() );
		event = genEntry.getEntry();
		fields = genEntry.getAll();
		
		event.setTitle( ( '*' + fields["title"] + '*' ) );
		event.setContent( google.gdata.atom.Text.create( fields["desc"] ) );
		
		if( fields["recurrent"] ) 
			var rRule = this.recParser.build( fields );
		ddiv( '#msgc', 'Credentials: <br>' + flatkv( this.calendarService.getUserCredentials() ), 'bgblue' );
		ddiv( '#debug_models', event.getTitle().getText(), 'notifs' );
		ddiv( '#debug_models', flatkv( fields ), 'notifs', false );
		ddiv( '#debug_models', flatkv( event ), 'notifs', false );
		newEvent = new google.gdata.calendar.CalendarEventEntry();
		newEvent.setTitle( event.getTitle() );
		
		myOkHandler = function( eRoot ) { ddiv( '#messages', 'Goooooooooooaaaaaaaaaaaaallllllll!', 'bggreen', false ); };
		myErrHandler = function( e ) { ddiv( '#messages', e.cause ? e.cause.statusText : e.message, 'bgred', false ); };
//		this.calendarService.insertEntry( this.eventFeedUrl, newEvent, this.entryCreated, this.handleError);
		this.calendarService.updateEntry( this.eventFeedUrl, event, myOkHandler, myErrHandler );
	},
	
	/**
	 * Called when event is added.
	 */
	entryCreated : function() {
	  mdiv( 'Successfully added to calendar.', 'bggreen' );
	},

	/** Called if an error is encountered while retrieving a feed or adding an event. */
	handleError : function(msg) {
		mdiv( msg, 'bgred' );
	}

}
);

